import click
import os

from src.data.bio_loader import BioLoaders
from src.evaluate.biological import BiologicalEvaluator


@click.command()
@click.argument('experiment', type=click.Path(exists=True))
@click.option('--show/--no-show', default=True, help="show matching")
def main(experiment, show):
    folder, filename = os.path.split(experiment)
    dataset = filename.split("_experiments.json")[0]
    evaluator = BiologicalEvaluator(dataset)
    
    evaluator.load_experiments(exp=experiment)
    
    savefolder = os.path.join(folder, "figures")
    evaluator.create_figure(folder=savefolder, show=show)


@click.command()
@click.argument('folder', type=click.Path(exists=True))
@click.option('--show/--no-show', default=True, help="show matching")
def mainall(folder, show):
    for dataset in BioLoaders.keys():
        evaluator = BiologicalEvaluator(dataset)
        experiment = os.path.join(folder, "{}_experiments.json".format(dataset))
        try:
            evaluator.load_experiments(exp=experiment)

            savefolder = os.path.join(folder, "figures")
            evaluator.create_figure(folder=savefolder, show=show)
        except FileNotFoundError:
            continue
    

if __name__ == "__main__":
    mainall()
